@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.5.  */
+/* A Bison parser, made by GNU Bison 3.3.2.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015, 2018 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
 #ifndef YY_I386_I_PARSE_H_INCLUDED
 # define YY_I386_I_PARSE_H_INCLUDED
 /* Debug traces.  */
@@ -74,7 +78,7 @@ extern int i386_debug;
 
 union YYSTYPE
 {
-#line 216 "i386_parse.y" /* yacc.c:1910  */
+#line 216 "i386_parse.y" /* yacc.c:1921  */
 
   unsigned long int num;
   char *str;
@@ -84,7 +88,7 @@ union YYSTYPE
   struct argname *name;
   struct argument *arg;
 
-#line 88 "i386_parse.h" /* yacc.c:1910  */
+#line 92 "i386_parse.h" /* yacc.c:1921  */
 };
 
 typedef union YYSTYPE YYSTYPE;
