@@ -1,5 +1,5 @@
 /* Compute simple checksum from permanent parts of the ELF file.
-   Copyright (C) 2002, 2003, 2004, 2005, 2009 Red Hat, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005, 2009, 2015 Red Hat, Inc.
    This file is part of elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 2002.
 
@@ -46,18 +46,12 @@
 #endif
 
 
-/* The SECTION_STRIP_P macro wants to call into libebl which we cannot
-   do and do not have to do here.  Provide a dummy replacement.  */
-#define ebl_debugscn_p(ebl, name) true
-
-
 #define process_block(crc, data) \
   __libelf_crc32 (crc, data->d_buf, data->d_size)
 
 
 long int
-elfw2(LIBELFBITS,checksum) (elf)
-     Elf *elf;
+elfw2(LIBELFBITS,checksum) (Elf *elf)
 {
   size_t shstrndx;
   Elf_Scn *scn;
@@ -121,7 +115,7 @@ elfw2(LIBELFBITS,checksum) (elf)
 
       /* To compute the checksum we need to get to the data.  For
 	 repeatable results we must use the external format.  The data
-	 we get with 'elf'getdata' might be changed for endianess
+	 we get with 'elf'getdata' might be changed for endianness
 	 reasons.  Therefore we use 'elf_rawdata' if possible.  But
 	 this function can fail if the data was constructed by the
 	 program.  In this case we have to use 'elf_getdata' and
